==> mir_0.21.0%2B16.04.20160330.orig.tar.gz: src/server/compositor/screencast_display_buffer.h <==
/*
 * Copyright © 2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
 */

#ifndef MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_
#define MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_

#include "mir/graphics/display_buffer.h"
#include "mir/renderer/gl/render_target.h"

#include <GLES2/gl2.h>

namespace mir
{
namespace renderer { namespace gl { class TextureSource; }}
namespace compositor
{
namespace detail
{

template <void (*Generate)(GLsizei,GLuint*), void (*Delete)(GLsizei,GLuint const*)>
class GLResource
{
public:
    GLResource() { Generate(1, &resource); }
    ~GLResource() { Delete(1, &resource); }
    operator GLuint() const { return resource; }

private:
    GLResource(GLResource const&) = delete;
    GLResource& operator=(GLResource const&) = delete;
    GLuint resource = 0;
};
}

class ScreencastDisplayBuffer : public graphics::DisplayBuffer,
                                public graphics::NativeDisplayBuffer,
                                public renderer::gl::RenderTarget
{
public:
    ScreencastDisplayBuffer(
        geometry::Rectangle const& rect,
        graphics::Buffer& buffer);
    ~ScreencastDisplayBuffer();

    geometry::Rectangle view_area() const override;

    void make_current() override;

    void release_current() override;

    bool post_renderables_if_optimizable(graphics::RenderableList const&) override;

    void swap_buffers() override;

    MirOrientation orientation() const override;

    NativeDisplayBuffer* native_display_buffer() override;

private:
    geometry::Rectangle const rect;
    graphics::Buffer& buffer;
    renderer::gl::TextureSource* texture_source;
    GLint old_fbo;
    GLint old_viewport[4];
    detail::GLResource<glGenTextures,glDeleteTextures> const color_tex;
    detail::GLResource<glGenRenderbuffers,glDeleteRenderbuffers> const depth_rbo;
    detail::GLResource<glGenFramebuffers,glDeleteFramebuffers> const fbo;
};

}
}

#endif /* MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_ */

==> mir_0.24.0%2B16.10.20160815.3.orig.tar.gz: src/server/compositor/screencast_display_buffer.h <==
/*
 * Copyright © 2014 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alexandros Frantzis <alexandros.frantzis@canonical.com>
 */

#ifndef MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_
#define MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_

#include "mir/graphics/display_buffer.h"
#include "mir/renderer/gl/render_target.h"

#include MIR_SERVER_GL_H
#include MIR_SERVER_GLEXT_H

namespace mir
{
namespace graphics
{
class Display;
class GLContext;
}
namespace renderer { namespace gl { class TextureSource; }}
namespace compositor
{

namespace detail
{

template <void (*Delete)(GLsizei,GLuint const*)>
class GLResource
{
public:
    GLResource() {}
    GLResource(GLuint resource) : resource{resource} {}
    ~GLResource() { reset(); }

    operator GLuint() const { return resource; }

    void reset()
    {
        if (resource)
        {
            Delete(1, &resource);
            resource = 0;
        }
    }

    GLResource(GLResource&& other) : resource {other.resource}
    {
        other.resource = 0;
    }

    GLResource& operator=(GLResource&& other)
    {
        resource = other.resource;
        other.resource = 0;
        return *this;
    }

private:
    GLResource(GLResource const&) = delete;
    GLResource& operator=(GLResource const&) = delete;
    GLuint resource{0};
};
}

class Schedule;
class ScreencastDisplayBuffer : public graphics::DisplayBuffer,
                                public graphics::NativeDisplayBuffer,
                                public renderer::gl::RenderTarget
{
public:
    ScreencastDisplayBuffer(
        geometry::Rectangle const& rect,
        geometry::Size const& size,
        MirMirrorMode mirror_mode,
        Schedule& free_queue,
        Schedule& ready_queue,
        graphics::Display& display);
    ~ScreencastDisplayBuffer();

    geometry::Rectangle view_area() const override;

    void make_current() override;

    void bind() override;

    void release_current() override;

    bool post_renderables_if_optimizable(graphics::RenderableList const&) override;

    void swap_buffers() override;

    MirOrientation orientation() const override;

    MirMirrorMode mirror_mode() const override;

    NativeDisplayBuffer* native_display_buffer() override;

private:
    std::unique_ptr<graphics::GLContext> gl_context;
    geometry::Rectangle const rect;
    MirMirrorMode const mirror_mode_;

    Schedule& free_queue;
    Schedule& ready_queue;
    std::shared_ptr<graphics::Buffer> current_buffer;

    GLint old_fbo;
    GLint old_viewport[4];

    detail::GLResource<glDeleteTextures> color_tex;
    detail::GLResource<glDeleteRenderbuffers> depth_rbo;
    detail::GLResource<glDeleteFramebuffers> fbo;
};

}
}

#endif /* MIR_COMPOSITOR_SCREENCAST_DISPLAY_BUFFER_H_ */
